#ifndef __AMSSTRING_HPP__
#define __AMSSTRING_HPP__

//This is a mutable C++ string class
//everyone's got one.
//this one is mine

//depends on stdarg.h for variable argument functions
//depends on std::vector and std::string
//depends on string.h for c string functions

namespace ams
{

typedef char ams_chartype;
static ams_chartype ams_char_cr = '\r'; //carriage return
static ams_chartype ams_char_lf = '\n'; //newline
static ams_chartype ams_char_tb = '\t'; //tab
static ams_chartype ams_char_nt = '\0'; //null terminator

class amsstring
{
public:
  ams_chartype *cstring;
  int length; //length will be set to the length of the cstring not including the null terminating char


  amsstring();
  amsstring(int length);
  amsstring(int length, const ams_chartype initchar);
  amsstring(const ams_chartype *inpstr);
  ~amsstring();

  //copy constructor (necessary for using container classes)
  //also copy assignment operator (rule of three)
  amsstring(const amsstring &other);
  amsstring& operator=(const amsstring &other);

  amsstring& operator=(const char *other); //assign string constant to amsstring

  //rule of 5 requires move and move assignment operators (more magic??)


  //string comparisons
  bool operator==(const amsstring &other);
  bool operator==(const char *other);
  bool operator!=(const amsstring &other);
  bool operator!=(const char *other);

//string ordering comparison
//alphebetizes strings by ASCII character
//longer strings campare larger than shorter ones
  bool operator<(const amsstring &other);
  bool operator>(const amsstring &other);
  bool operator<(const ams_chartype *other);
  bool operator>(const ams_chartype *other);

  ams_chartype& operator[](unsigned int ind);

  void clear();
  void setblank();

  const ams_chartype * c_str();


  void copy(const amsstring *other);
  void copy(amsstring *other);
  void copy(std::string *other);
  void copy(const ams_chartype *other);
  int size();
  void resize(int N);

  void substring(int ind1, int ind2, amsstring *sout);
  char& at(int ind);
  void insert(int ind, amsstring *strtoins);
  void insert(int ind, const ams_chartype *strtoins);
  void insert(int ind, amsstring strtoins);
  void remove(int ind);
  void remove(int ind1, int ind2);

  void append(const ams_chartype c); //appends single character
  void append(amsstring *s);
  void append(const ams_chartype *s);
  void append(amsstring s);
  void append(int i);
  void append(double d);

  int find(amsstring substr, int indstart=0, bool casesens=1);
  int find(const ams_chartype c, int indstart=0, bool casesens=1);
  int find(amsstring *substr, int indstart=0, bool casesens=1);
  int find(const ams_chartype *str, int indstart=0, bool casesens=1);

  int sprintf(int bufflen, const ams_chartype *formatstring, ...);

  bool isvalidnumber();
  void tolower();
  void toupper();

  double str2double();
  int str2int();


  //operator overloads

};


//all functions dealing with amsstring will have two to three versions:
//1. One dealing with ANSI C char arrays
//2. One dealing with std::strings
//3. One dealing with the mutable amsstrings

//I want to

void tolower(amsstring *s);
void toupper(amsstring *s);

//strneq works the way a string comparator should work in my mind
//if the two strings are THE SAME LENGTH and have THE SAME CHARACTERS
//true is returned. Otherwise false. Take notes MATLAB!
bool strneq(const amsstring *s1, const amsstring *s2, bool casesens=1);
bool strneq(const amsstring s1, const amsstring s2, bool casesens=1);
bool strneq(const amsstring s1, const ams_chartype *s2, bool casesens=1);
bool strneq(const amsstring *s1, const ams_chartype *s2, bool casesens=1);
bool strneq(const ams_chartype *s1, const ams_chartype *s2, bool casesens=1);

void splitlines(amsstring *s, std::vector<amsstring> *lns);

void split(amsstring *s, const ams_chartype delimitchar, std::vector<amsstring> *lns);
void split(amsstring *s, const ams_chartype *delimitstr, std::vector<amsstring> *lns);
void split(amsstring *s, amsstring *delimitstr, std::vector<amsstring> *lns);

//splits a string, not counting whitespaces between non-whitespace characters
void splitwhitespace(amsstring *s, std::vector<amsstring> *lns);

//removes all whitespace characters '\t','\r','\n' included
//to the left and right of the string (but not in the middle)
void stripwhitespace(amsstring *s);

//completely removes all whitespace entirely
void stripallwhitespace(amsstring *s);


double str2double(amsstring *s);
int str2int(amsstring *s);

void freadline(FILE *fp,amsstring *s);
void freadlines(FILE *fp, std::vector<amsstring> *lines);
void fwritelines(FILE *fp,amsstring *s);
void freadtxtfile(FILE *fp, amsstring *s);

//other utility functions

//takes ASCII formatted string and converts it to a double precision floating point value
//also handles a few other things: nans and infs
//anything that it cannot parse, it returns (without throwing an error), as a nan
double amsstrtod(const char *str);

void test_amsstrtod();

//takes ASCII formatted string and checks for numeric validity
//nans given as nan count (even though they're not a number)
//+/- infinity counts
//anything else does not count
bool ams_isvalidnumber(const char *str);

//Portability glue
int amsstrcpy_s(char *dest, int size, const char *src);
int amssprintf_s(char *s, int n, const char *format, ...);


};

#endif
